package com.ruoyi.web.controller.common;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.QRCodeUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.file.ImageUtils;
import com.ruoyi.common.utils.file.OssUtils;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.generator.domain.Community.CommunityArticleImages;
import com.ruoyi.generator.mapper.community.CommunityArticleImagesMapper;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

/**
 * 通用请求处理
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/common")
public class CommonController {
    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;

    private static final String FILE_DELIMETER = ",";

    private static final String fileUrl = "http://file.iciyuanshidai.com";

    @Resource
    private CommunityArticleImagesMapper communityArticleImagesMapper;

    /**
     * 通用下载请求
     *
     * @param fileName 文件名称
     * @param delete   是否删除
     */
    @GetMapping("/download")
    public void fileDownload(String fileName, Boolean delete, HttpServletResponse response, HttpServletRequest request) {
        try {
            if (!FileUtils.checkAllowDownload(fileName)) {
                throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
            }
            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
            String filePath = RuoYiConfig.getDownloadPath() + fileName;

            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, realFileName);
            FileUtils.writeBytes(filePath, response.getOutputStream());
            if (delete) {
                FileUtils.deleteFile(filePath);
            }
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

//    /**
//     * 通用上传请求（单个）
//     */
//    @PostMapping("/upload")
//    public AjaxResult uploadFile(MultipartFile file) throws Exception {
//        try {
//            // 上传文件路径
//            String filePath = RuoYiConfig.getUploadPath();
//            String thumbnailPath = "";
//            // 上传并返回新文件名称
//            String fileName = FileUploadUtils.upload(filePath, file);
//            String url = serverConfig.getUrl() + fileName;
//            String fileType = FileUtils.getFileType(file);
//            if ("video".equalsIgnoreCase(fileType)) {
//                thumbnailPath = fileName.replace("/profile", RuoYiConfig.getProfile());
//                int dotIndex = thumbnailPath.lastIndexOf(".");
//                if (dotIndex != -1) {
//                    thumbnailPath = thumbnailPath.substring(0, dotIndex) + ".jpg";
//                } else {
//                    thumbnailPath = thumbnailPath + ".jpg";  // 如果没有后缀，直接加 .jpg
//                }
//                FileUtils.screenShots(fileName.replace("/profile", RuoYiConfig.getProfile()), thumbnailPath, "00:00:01");
//            }
//
//            if ("image".equalsIgnoreCase(fileType)) {
//                //获取用户名
//                String nickName = SecurityUtils.getLoginUser().getUser().getNickName();
//                // 原始文件
//                if (fileName.contains("/profile/upload")) {
//                    fileName = fileName.replace("/profile/upload", "");
//                }
//
//                File sourceFile = new File(filePath + fileName);
//                //图片添加水印
//                String watermarkFileName = "watermark_" + fileName;
//                File destFile = new File(filePath + fileName);
//                // 加水印
//                ImageUtils.addWatermark(sourceFile, destFile, "@" + nickName, "次元时代-ACG爱好者社区");
//                fileName = watermarkFileName;
//            }
//
//            AjaxResult ajax = AjaxResult.success();
//            ajax.put("url", url);
//            ajax.put("fileName", fileName);
//            ajax.put("newFileName", FileUtils.getName(fileName));
//            ajax.put("originalFilename", file.getOriginalFilename());
//            ajax.put("fileType", fileType);
//            ajax.put("thumbnailPath", thumbnailPath);
//            return ajax;
//        } catch (Exception e) {
//            return AjaxResult.error(e.getMessage());
//        }
//    }

    /**
     * 通用上传请求（单个） OSS
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file, boolean waterMark, boolean isFillWater) throws Exception {
        try {
            String fileType = FileUtils.getFileType(file);
//            if ("image".equalsIgnoreCase(fileType)) {
//                file = FileUtils.convertToJpg(file);
//            }
            // 上传文件路径
            String fileName = OssUtils.uploadMultipartFile(file);
            System.out.println(fileName);
            String fileNameResult = fileName.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl);
            String thumbnailPath = "";
            String compressedFileUrl = "";

            if ("video".equalsIgnoreCase(fileType) && waterMark) {
                String tempThumbnailFile = System.getProperty("java.io.tmpdir") + File.separator + "thumbnail.jpg";

                boolean screenshotSuccess = FileUtils.screenShots(fileNameResult, tempThumbnailFile, "00:00:01");
                if (!screenshotSuccess) {
                    throw new RuntimeException("生成视频缩略图失败");
                }

                // 上传缩略图到OSS
                thumbnailPath = OssUtils.uploadFile(tempThumbnailFile);

                // 删除临时缩略图文件
                new File(tempThumbnailFile).delete();
            }

            if ("image".equalsIgnoreCase(fileType)) {
                String tempWatermarkFile = "";
                // 下载源文件到本地临时目录
                String tempSourceFile = System.getProperty("java.io.tmpdir") + File.separator + "source_" + FileUtils.getName(fileName);
                OssUtils.downloadFile(fileName.replace("https://cysd.oss-cn-shanghai.aliyuncs.com/", ""), tempSourceFile);
                if (waterMark) {
                    //获取用户名
                    String nickName = SecurityUtils.getLoginUser().getUser().getNickName();
                    // 本地临时水印文件路径
                    tempWatermarkFile = System.getProperty("java.io.tmpdir") + File.separator + "watermark_" + FileUtils.getName(fileName);
                    // 添加水印
                    ImageUtils.addWatermark(
                            isFillWater,
                            fileName.replace("https://cysd.oss-cn-shanghai.aliyuncs.com/", ""),
                            new File(tempSourceFile),
                            new File(tempWatermarkFile),
                            "@" + nickName,
                            "ACG爱好者社区"
                    );
                    // 上传加水印的文件到OSS
                    fileNameResult = OssUtils.uploadFile(tempWatermarkFile);
                }

                //缩略图上传
                String compressedFile = System.getProperty("java.io.tmpdir") + File.separator + "compress_" + FileUtils.getName(fileName);
                ImageUtils.compressAndGenerateThumbnail(tempSourceFile, compressedFile, 500, 350, 0.8f);
                compressedFileUrl = OssUtils.uploadFile(compressedFile);

                // 删除临时文件
                if (new File(tempSourceFile).exists()) {
                    new File(tempSourceFile).delete();
                }

                if (new File(compressedFile).exists()) {
                    new File(compressedFile).delete();
                }

                if (new File(tempWatermarkFile).exists()) {
                    new File(tempWatermarkFile).delete();
                }
            }

            fileNameResult = fileNameResult.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl);
            compressedFileUrl = compressedFileUrl.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl);
            String url = fileNameResult;
            AjaxResult ajax = AjaxResult.success();
            ajax.put("url", url);
            ajax.put("fileName", fileNameResult);
            ajax.put("newFileName", FileUtils.getName(fileNameResult));
            ajax.put("originalFilename", file.getOriginalFilename());
            ajax.put("compressedFileUrl", compressedFileUrl);
            ajax.put("fileType", fileType);
            ajax.put("thumbnailPath", thumbnailPath.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl));
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }


    @PostMapping("/change")
    public AjaxResult changeMethod() throws Exception {
        // 1. 取出图片
        List<CommunityArticleImages> images = communityArticleImagesMapper.selectList(new QueryWrapper<CommunityArticleImages>()
                .isNotNull("image_url")
                .like("image_url", "47.122.10.161:8084"));

        for (CommunityArticleImages image : images) {
            // 2. 对URL中的路径部分进行编码
            String imageUrl = image.getImageUrl();
            if (imageUrl.contains(".mp4")) {
                continue;
            }
            String encodedImageUrl = encodeUrl(imageUrl); // 编码 URL 路径部分

            // 3. 获取文件类型（图片、音频、视频等）
            String fileType = FileUtils.getFileTypeFromUrl(encodedImageUrl);

            // 根据文件类型处理
            String tempUrl = "";
            if ("image".equals(fileType)) {
                // 处理图片
                tempUrl = processImage(encodedImageUrl, image);
            } else {
                // 如果不是图片，跳过处理
                System.out.println("Skipping non-image file: " + imageUrl);
            }
            //如果是服务器图片,则更新图片信息
            if (imageUrl.contains("47.122.10.161:8084")) {
                String resultUrl = OssUtils.uploadFile(tempUrl);
                image.setImageUrl(resultUrl.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl));
                communityArticleImagesMapper.updateById(image);

                //删除临时的原文件
                org.apache.commons.io.FileUtils.deleteQuietly(new File(tempUrl));
            }
        }
        return AjaxResult.success();
    }

    /**
     * 处理图片：下载、压缩、上传到OSS
     */
    private String processImage(String imageUrl, CommunityArticleImages image) throws Exception {
        // 下载图片到临时文件
        String tempUrl = System.getProperty("java.io.tmpdir") + File.separator + "original_" + image.getImageUrl().substring(image.getImageUrl().lastIndexOf("/") + 1);
        File tempFile = new File(tempUrl);
        URL url = new URL(imageUrl);
        try (InputStream inputStream = url.openStream();
             OutputStream outputStream = new FileOutputStream(tempFile)) {
            IOUtils.copy(inputStream, outputStream);
        }

        // 4. 将图片进行压缩
        String compressedFile = System.getProperty("java.io.tmpdir") + File.separator + "compress_" + image.getImageUrl().substring(image.getImageUrl().lastIndexOf("/") + 1);
        ImageUtils.compressAndGenerateThumbnail(tempFile.getAbsolutePath(), compressedFile, 500, 350, 0.8f);

        // 5. 将压缩后的图片上传到OSS
        String compressedFileUrl = OssUtils.uploadFile(compressedFile);

        // 6. 更新数据库
        image.setCompressUrl(compressedFileUrl.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl));
        communityArticleImagesMapper.updateById(image);

        // 删除临时文件
        org.apache.commons.io.FileUtils.deleteQuietly(new File(compressedFile));  // 删除压缩后的临时文件

        return tempUrl;
    }


    /**
     * 对URL中的中文字符进行编码，同时避免对协议部分（http:// 或 https://）和主机部分编码
     */
    private String encodeUrl(String url) throws UnsupportedEncodingException {
        // 检查协议部分并分离
        String protocol = "";
        if (url.startsWith("http://")) {
            protocol = "http://";
            url = url.substring(7);
        } else if (url.startsWith("https://")) {
            protocol = "https://";
            url = url.substring(8);
        }

        // 分离主机和端口部分
        int pathStartIndex = url.indexOf("/");
        String hostPart = url;
        String pathPart = "";
        if (pathStartIndex > -1) {
            hostPart = url.substring(0, pathStartIndex); // 主机部分（包括端口号）
            pathPart = url.substring(pathStartIndex);   // 路径部分
        }

        // 确保端口号部分不被错误编码，只有路径部分需要编码
        String encodedPath = URLEncoder.encode(pathPart, "UTF-8");
        // 保留斜杠和特定字符不被编码
        encodedPath = encodedPath.replace("%2F", "/"); // 保留斜杠
        // 确保空格被编码为 %20，而不是 +
        encodedPath = encodedPath.replace("+", "%20");
        encodedPath = encodedPath.replace("%28", "(").replace("%29", ")"); // 保留括号
        encodedPath = encodedPath.replace("%2B", "+"); // 保留加号
        // 返回完整的 URL
        return protocol + hostPart + encodedPath;
    }

    /**
     * 通用上传请求（多个）
     */
    @PostMapping("/uploads")
    public AjaxResult uploadFiles(List<MultipartFile> files) throws Exception {
        try {
            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            List<String> urls = new ArrayList<String>();
            List<String> fileNames = new ArrayList<String>();
            List<String> newFileNames = new ArrayList<String>();
            List<String> originalFilenames = new ArrayList<String>();
            for (MultipartFile file : files) {
                // 上传并返回新文件名称
                String fileName = FileUploadUtils.upload(filePath, file);
                String url = serverConfig.getUrl() + fileName;
                urls.add(url);
                fileNames.add(fileName);
                newFileNames.add(FileUtils.getName(fileName));
                originalFilenames.add(file.getOriginalFilename());
            }
            AjaxResult ajax = AjaxResult.success();
            ajax.put("urls", StringUtils.join(urls, FILE_DELIMETER));
            ajax.put("fileNames", StringUtils.join(fileNames, FILE_DELIMETER));
            ajax.put("newFileNames", StringUtils.join(newFileNames, FILE_DELIMETER));
            ajax.put("originalFilenames", StringUtils.join(originalFilenames, FILE_DELIMETER));
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 本地资源通用下载
     */
    @GetMapping("/download/resource")
    public void resourceDownload(String resource, HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        try {
            if (!FileUtils.checkAllowDownload(resource)) {
                throw new Exception(StringUtils.format("资源文件({})非法，不允许下载。 ", resource));
            }
            // 本地资源路径
            String localPath = RuoYiConfig.getProfile();
            // 数据库资源地址
            String downloadPath = localPath + StringUtils.substringAfter(resource, Constants.RESOURCE_PREFIX);
            // 下载名称
            String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, downloadName);
            FileUtils.writeBytes(downloadPath, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    /**
     * 生成二维码
     *
     * @param
     * @return
     */
    @GetMapping("/loginUserQRCode")
    public AjaxResult generateAndUploadQRCode(boolean isReturn) {
        String fileName = "";
        try {
            Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
            JSONObject userIdObj = new JSONObject();
            userIdObj.put("userId", userId);
            if (isReturn) {
                userIdObj.put("fantuFlag", true);
            }
            // 1. 生成临时二维码图片
            fileName = "qrcode_" + userId + ".png";
            Path tempFile = Paths.get(System.getProperty("java.io.tmpdir"), fileName);
            QRCodeUtils.generateQRCode(userIdObj.toJSONString(), tempFile, 300, 300);

            // 2. 转为 MultipartFile 格式，便于调用 RuoYi 的上传接口
            File file = tempFile.toFile();
            MultipartFile multipartFile = FileUtils.convertFileToMultipartFile(file);

            // 3. 使用 RuoYi 的文件上传工具进行上传
            String uploadPath = FileUploadUtils.upload(RuoYiConfig.getUploadPath(), multipartFile);

            // 4. 拼接访问 URL 并返回给前端
            String fileUrl = serverConfig.getUrl() + uploadPath;
            return AjaxResult.success("二维码生成成功", fileUrl);

        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error("二维码生成或上传失败");
        } finally {
            //清理临时文件
            FileUtils.cleanTempDir(fileName);
        }
    }

}
