package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.CommunityTagBlockMapper;
import com.ruoyi.generator.mapper.community.CommunityTagMapper;
import com.ruoyi.generator.mapper.community.CommunityUserCircleMapper;
import com.ruoyi.generator.mapper.community.CommunityUserTagMapper;
import com.ruoyi.generator.service.ICommunityTagService;
import com.ruoyi.generator.vo.CommunityTagVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 文章标签
 *
 * @author fangzhen
 */
@Api(tags = "文章标签")
@RestController
@RequestMapping("/community/tag")
public class CommunityTagController extends BaseController {

    @Autowired
    private ICommunityTagService communityTagService;

    @Autowired
    private CommunityTagBlockMapper communityTagBlockMapper;

    @Autowired
    private CommunityTagMapper communityTagMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private CommunityUserCircleMapper communityUserCircleMapper;

    @Autowired
    private CommunityUserTagMapper communityUserTagMapper;
    /**
     * 获取标签信息
     */
    @ApiOperation("获取标签信息")
    @GetMapping()
    @Transactional
    //@Anonymous
    public AjaxResult tags(String tagName) {
        List<CommunityTag> communityTags = null;
        try {
            int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
            int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
            int offset = (pageNum - 1) * pageSize;
            Page<CommunityTag> page = new Page<>(offset, pageSize);
            communityTags = communityTagService.page(page, new QueryWrapper<CommunityTag>().and((wrapper) -> {
                        wrapper.ne("is_delete", 1)
                                .or()
                                .isNull("is_delete");
                    })
                    .like("tag_name", tagName)
                    .orderByDesc("tag_hot")).getRecords();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityTags);
    }

    /**
     * 获取我创建的标签
     */
    @ApiOperation("我创建的标签")
    @GetMapping("/createdTags")
    @Transactional
    //@Anonymous
    public AjaxResult createdTags(Long userId) {
        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }
        List<CommunityTag> communityTags = null;
        try {
            int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
            int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
            int offset = (pageNum - 1) * pageSize;
            Page<CommunityTag> page = new Page<>(offset, pageSize);
            communityTags = communityTagService.page(page, new QueryWrapper<CommunityTag>().and((wrapper) -> {
                        wrapper.ne("is_delete", 1)
                                .or()
                                .isNull("is_delete");
                    })
                    .eq("create_by", userId)
                    .orderByDesc("tag_hot")).getRecords();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityTags);
    }


    @ApiOperation("收藏/取关标签")
    @PostMapping("/likeTag")
    @Transactional
    //@Anonymous
    public AjaxResult likeTag(@RequestBody CommunityUserTag communityUserTag) {
        if (Objects.isNull(communityUserTag.getTagId())) {
            return AjaxResult.error("收藏标签异常,请刷新页面重试!");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        String msg = "";
        try {
            CommunityUserTag userTag = communityUserTagMapper
                    .selectOne(new QueryWrapper<CommunityUserTag>()
                            .eq("tag_id", communityUserTag.getTagId())
                            .eq("user_id", userId)
                            .eq("is_delete", 0));

            if (Objects.isNull(userTag) || Objects.isNull(userTag.getId())) {
                //如果未找到数据,则进行关注
                userTag = new CommunityUserTag();
                userTag.setTagId(communityUserTag.getTagId());
                userTag.setUserId(userId);
                userTag.setCreateBy(userId);
                userTag.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityUserTagMapper.insert(userTag);
                msg = "已关注";
            } else {
                //更新删除标识
                userTag.setDelete(true);
                userTag.setUpdateBy(userId);
                userTag.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityUserTagMapper.updateById(userTag);
                msg = "已取关";
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(msg);
    }

    /**
     * 获取我关注的标签
     */
    @ApiOperation("我关注的标签")
    @GetMapping("/likeTags")
    @Transactional
    //@Anonymous
    public AjaxResult likeTags(Long userId) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int offset = (pageNum - 1) * pageSize;

        Page<CommunityUserTag> page = new Page<>(offset, pageSize);
        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }
        List<CommunityTag> communityTags = null;
        try {
            //获取关注标签ID
            QueryWrapper<CommunityUserTag> queryWrapper = new QueryWrapper<CommunityUserTag>()
                    .eq("user_id", userId)
                    .eq("is_delete", 0);
            List<CommunityUserTag> userTags = communityUserTagMapper
                    .selectPage(page, queryWrapper).getRecords();

            List<Long> tagList = userTags.stream().map(CommunityUserTag::getTagId).collect(Collectors.toList());
            if (tagList.size() == 0){
                return AjaxResult.success(communityTags);
            }

            communityTags= communityTagMapper.selectList(new QueryWrapper<CommunityTag>()
                    .in("id", tagList)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));


        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityTags);
    }

    /**
     * 获取标签信息(当前用户是否已拉黑)
     */
    @ApiOperation("获取标签信息(附带拉黑信息)")
    @GetMapping("/tagsTakeBlock")
    @Transactional
    //@Anonymous
    public AjaxResult tagsTakeBlock(Long userId) {
       /* List<CommunityTagVo> communityTagVos = new ArrayList<>();
        try {
            Page<CommunityTag> page = new Page<>(1, 10);
            List<CommunityTag> communityTags = communityTagService.page(page, new QueryWrapper<CommunityTag>().and((wrapper) -> {
                        wrapper.ne("is_delete", 1)
                                .or()
                                .isNull("is_delete");
                    })
                    .like("tag_name", tagName)
                    .orderByDesc("tag_hot")).getRecords();

            if (communityTags.isEmpty()){
                return AjaxResult.success("false");
            }
            CommunityTagVo communityTagVo = null;
            for (CommunityTag communityTag : communityTags) {
                communityTagVo = new CommunityTagVo();
                BeanUtils.copyProperties(communityTag, communityTagVo);
                communityTagVos.add(communityTagVo);
            }

            //查询是否被当前用户拉黑
            List<Long> tagIds = communityTagVos.stream().map(CommunityTagVo::getId).collect(Collectors.toList());
            Long userId = SecurityUtils.getUserId();
            List<CommunityTagBlock> communityTagBlocks = communityTagBlockMapper.selectList(new QueryWrapper<CommunityTagBlock>().eq("user_id", userId).in("tag_id", tagIds));

            //设置是否已拉黑
            for (CommunityTagVo tagVo : communityTagVos) {
                Long id = tagVo.getId();
                for (CommunityTagBlock communityTagBlock : communityTagBlocks) {
                    if (communityTagBlock.getTagId().equals(id)) {
                        tagVo.setBlock(communityTagBlock.isBlock());
                        break;
                    }
                }
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
*/
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int offset = (pageNum - 1) * pageSize;

        Page<CommunityTagBlock> page = new Page<>(offset, pageSize);
        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }
        List<CommunityTag> communityTags = null;
        try {
            //获取拉黑标签ID
            QueryWrapper<CommunityTagBlock> queryWrapper = new QueryWrapper<CommunityTagBlock>()
                    .eq("user_id", userId)
                    .eq("is_block", 1);

            List<CommunityTagBlock> userTags = communityTagBlockMapper.selectPage(page, queryWrapper).getRecords();

            List<Long> tagList = userTags.stream().map(CommunityTagBlock::getTagId).collect(Collectors.toList());
            if (tagList.size() == 0){
                return AjaxResult.success(communityTags);
            }

            communityTags= communityTagMapper.selectList(new QueryWrapper<CommunityTag>()
                    .in("id", tagList)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));


        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityTags);
    }

    /**
     * 新增标签
     */
    @ApiOperation("新增标签")
    @PostMapping("/addTag")
    @Transactional
    //@Anonymous
    public AjaxResult addTag(String tagName,String attribute1,String attribute2,String attribute3,String attribute4,String attribute5) {
        if (StringUtils.isEmpty(tagName)) {
            return AjaxResult.error("参数异常!");
        }

        try {
            CommunityTag communityTag = new CommunityTag();
            Long userId = SecurityUtils.getUserId();
            communityTag.setTagName(tagName.replaceAll("(?<=\\p{L})\\s+(?=\\p{N})|(?<=\\p{N})\\s+(?=\\p{L})|(?<=\\p{L})\\s+(?=\\p{L})", ""));
            communityTag.setTagHot(0L);
            communityTag.setDelete(false);
            communityTag.setCreateBy(userId);
            communityTag.setAttribute1(attribute1);
            communityTag.setAttribute2(attribute2);
            communityTag.setAttribute3(attribute3);
            communityTag.setAttribute4(attribute4);
            communityTag.setAttribute5(attribute5);
            communityTag.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            System.out.println("communityTag: " + communityTag);

            //判断标签名称是否重复
            CommunityTag CommunityTag2 = communityTagMapper.selectOne(new QueryWrapper<CommunityTag>()
                    .eq("tag_name", communityTag.getTagName())
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    }));
            if (CommunityTag2 != null) {
                return AjaxResult.error("标签名称重复");
            }
            communityTagService.save(communityTag);
//            redisCache.setCacheList(CacheConstants.COMMUNITY_TAG, communityTagMapper
//                    .selectList(new QueryWrapper<CommunityTag>()
//                    .eq("is_delete", false)));
//            redisCache.expire(CacheConstants.COMMUNITY_TAG, 24, TimeUnit.HOURS);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success("添加标签成功!");
    }

    /**
     * 拉黑标签
     */
    @ApiOperation("拉黑标签")
    @PostMapping("/blockTag")
    @Transactional
    //@Anonymous
    public AjaxResult blockTag(@RequestBody CommunityTagBlock tagBlock) {
        if (Objects.isNull(tagBlock.getTagId()) || Objects.isNull(tagBlock.getUserId())) {
            return AjaxResult.error("参数异常!");
        }

        CommunityTagBlock communityTagBlock = null;
        try {
            communityTagBlock = communityTagService.blockTag(tagBlock);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
//        redisCache.setCacheList(CacheConstants.COMMUNITY_TAG_BLOCK, communityTagBlockMapper.selectList(new QueryWrapper<CommunityTagBlock>()));
//        redisCache.expire(CacheConstants.COMMUNITY_TAG_BLOCK, 24, TimeUnit.HOURS);
        return AjaxResult.success(communityTagBlock);
    }
}
